@@ -11,16 +11,19 @@ function(Z,EZ,rho,Y,YL,odmax,odobs)
   lt<-lower.tri(Z)
   rws<-outer(1:nrow(Z),rep(1,nrow(Z)))
 
-  for(y in sample(c(0:ncol(YL))) )
+  Y[is.na(Y)]<- -1
+
+  for(y in sample(c( (-1):ncol(YL))) )
   {
     if(y<2)
     {
-      if(y==0){lbm<- rep(-Inf,nrow(Z))}
+      if(y<=0){lbm<- rep(-Inf,nrow(Z))}
       if(y==1){lbm<-pmax(0,apply(Z - (Y!=0)*(Inf^(Y!=0)),1,max,na.rm=TRUE)) }
     }
     if(y>=2) {lbm<-Z[cbind(1:nrow(Z),YL[,y-1])] }
-   
-    if(y<ncol(YL))  { ubm<- Z[ cbind(1:nrow(Z), YL[,y+1] )]  }
+  
+    if(y== -1) { ubm<-rep(Inf,nrow(Z))} 
+    if(y<ncol(YL) & y>=0)  { ubm<- Z[ cbind(1:nrow(Z), YL[,y+1] )]  }
     if(y==0) { ubm[odobs<odmax]<- 0 }
     if(y==ncol(YL)) { ubm<- rep(Inf,nrow(Z)) }
     ubm[is.na(ubm)]<-Inf ; lbm[is.na(lbm)]<- -Inf
@@ -28,14 +31,14 @@ function(Z,EZ,rho,Y,YL,odmax,odobs)
     for(k in sample(1:2)) 
     { 
     if(k==1) { 
-    up<- ut & Y==y
+    up<- ut & Y==y 
     rwb<-rws[up]
     lb<-lbm[rwb] ; ub<-ubm[rwb]
     ez<- EZ[up] + rho*( t(Z)[up]  - t(EZ)[up] )
     Z[up]<-ez+sz*qnorm(runif(sum(up),pnorm((lb-ez)/sz),pnorm((ub-ez)/sz)))
               }
     if(k==2)  { 
-    up<- lt & Y==y
+    up<- lt & Y==y 
     rwb<-rws[up]
     lb<-lbm[rwb] ; ub<-ubm[rwb]
     ez<- EZ[up] + rho*( t(Z)[up]  - t(EZ)[up] )
